@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2025 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -143,11 +143,23 @@ SCIP_RETCODE consdataFree(
    )
 {
    int nrows;
+   int i;
 
    assert( consdata != NULL );
    assert( *consdata != NULL );
 
    nrows = (*consdata)->nrows;
+
+   /* release variables in vars1 and vars2 array */
+   for (i = 0; i < nrows; ++i)
+   {
+      assert( (*consdata)->vars1[i] != NULL );
+      SCIP_CALL( SCIPreleaseVar(scip, &(*consdata)->vars1[i] ) );
+
+      assert( (*consdata)->vars2[i] != NULL );
+      SCIP_CALL( SCIPreleaseVar(scip, &(*consdata)->vars2[i] ) );
+   }
+
    SCIPfreeBlockMemoryArrayNull(scip, &((*consdata)->vars2), nrows);
    SCIPfreeBlockMemoryArrayNull(scip, &((*consdata)->vars1), nrows);
 
@@ -205,6 +217,13 @@ SCIP_RETCODE consdataCreate(
       }
    }
 
+   /* capture vars in vars1 and vars2 array */
+   for (i = 0; i < nrows; ++i)
+   {
+      SCIP_CALL( SCIPcaptureVar(scip, (*consdata)->vars1[i] ) );
+      SCIP_CALL( SCIPcaptureVar(scip, (*consdata)->vars2[i] ) );
+   }
+
    return SCIP_OKAY;
 }
 
